from os.path import join, splitext, split, basename, dirname
from shutil import make_archive


def unzip(zipped_file, dest_dir):
    '''Uncompress a .zip file to `dest_dir`.

    Make sure that python version > 2.7.4, otherwise this
    implementation is vulnerable to path traversal attacks.
    See:
      http://stackoverflow.com/a/12886818/1336939
    '''
    import zipfile
    with zipfile.ZipFile(zipped_file) as zf:
        zf.extractall(dest_dir)


def untar(tared_file, dest_dir):
    '''Uncompress a .tar file to `dest_dir`.'''
    import tarfile
    with tarfile.open(tared_file) as fin:
        fin.extractall(dest_dir)


def ungzip(gzipped_file, dest_dir):
    '''Uncompress a .gz file to `dest_dir`.'''
    import gzip
    fname, _ = splitext(basename(gzipped_file))
    dest_file = join(dest_dir, fname)
    with gzip.open(gzipped_file) as fin, open(dest_file, 'wb') as fout:
        fout.write(fin.read())


def unbz2(bz2ipped_file, dest_dir=None):
    '''Uncompress a .bz2 file to `dest_dir`.'''
    import bz2
    bz2_dir, bz2_fname = split(bz2ipped_file)
    fname, ext = splitext(bz2_fname)
    if dest_dir is None:
        dest_dir = bz2_dir
    with open(bz2ipped_file) as fin, open(join(dest_dir, fname), 'wb') as fout:
        fout.write(bz2.decompress(fin.read()))


def unxz(xzipped_file, dest_dir):
    '''Uncompress a .xz file to `dest_dir`.'''
    import contextlib, lzma, tarfile
    with contextlib.closing(lzma.LZMAFile(xzipped_file)) as xz:
        with tarfile.open(fileobj=xz) as f:
            f.extractall(dest_dir)


COMPRESSION_FORMATS = {'.zip': unzip,
                       '.gz' : ungzip,
                       '.bz2': unbz2,
                       '.xz' : unxz,
                       '.tar': untar
                      }


def unpack(compressed_file, dest_dir=None):
    '''Uncompress a file according to its extension.'''
    if dest_dir is None:
        dest_dir = dirname(compressed_file)
    fname, extension = splitext(basename(compressed_file))

    COMPRESSION_FORMATS[extension](compressed_file, dest_dir)

    # check in we have unpacked a tar file (eg, .tar.gz or .tar.bz2)
    unpacked_file = join(dest_dir, fname)
    _, extension = splitext(unpacked_file)
    if extension == '.tar':
        return unpack(unpacked_file, dest_dir)

    return dest_dir


def zipdir(dpath):
    make_archive(dpath, 'zip', dpath)
